<?php

namespace App\Http\Controllers;

use App\Models\Editor;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class EditorController extends Controller
{
    public function index()
    {
        $editors = Editor::all();

        return view('editors', compact('editors'));
    }

    public function store(Request $request): \Illuminate\Http\RedirectResponse
    {
        $request->validate([
                               'name' => 'required',
                               'company' => 'required',
                               'operating_system' => 'required',
                               'license' => 'required',
                               'logo' => 'required|mimes:jpeg,jpg,png,gif|required|max:2048',
                           ]);

        $editor = new Editor($request->except(['logo']));

        // save logo
        $fileName = Str::uuid() . '.' . $request->logo->getClientOriginalExtension();
        Storage::putFileAs('images/editors', $request->logo, $fileName, 'public');

        $editor->logo = $fileName;

        $editor->save();

        return redirect()->route('editors.index')
            ->with('success', 'Editor created successfully.');
    }
}
